@@ -511,6 +511,19 @@ no_ops            = {}'''.format(*self)
                       'upload'])
         self._dir.sort()
 
+    def _cleanup(self, endpoint):
+        """Attempt to cleanup in case of user interruption or error. Send a
+           cancel request to the query associated with the current
+           session
+
+        """
+        if endpoint == Shim.execute_query:
+            requests.get(
+                requests.compat.urljoin(self.scidb_url, Shim.cancel.value),
+                params={'id': self._id},
+                auth=self._http_auth,
+                verify=self.verify)
+
     def _shim(self, endpoint, **kwargs):
         """Make request on Shim endpoint"""
 
@@ -540,17 +553,15 @@ no_ops            = {}'''.format(*self)
                     auth=self._http_auth,
                     verify=self.verify)
             except KeyboardInterrupt as e:
-                if endpoint == Shim.execute_query:
-                    requests.get(
-                        requests.compat.urljoin(
-                            self.scidb_url, Shim.cancel.value),
-                        params={'id': self._id},
-                        auth=self._http_auth,
-                        verify=self.verify)
+                self._cleanup(endpoint)
                 raise e
 
         req.reason = req.content
-        req.raise_for_status()
+        try:
+            req.raise_for_status()
+        except Exception as e:
+            self._cleanup(endpoint)
+            raise e
         return req
 
     def _shim_readlines(self):
@@ -615,7 +626,7 @@ class Array(object):
     def __dir__(self):
         """Download the schema of the SciDB array, using ``show()``"""
         sh = Schema.fromstring(
-            self.db.iquery_readlines('show({})'.format(self))[0])
+            self.db.iquery_readlines('show({})'.format(self))[0][0])
         ls = [i.name for i in itertools.chain(sh.atts, sh.dims)]
         ls.sort()
         return ls
@@ -647,7 +658,7 @@ class Array(object):
 
     def schema(self):
         return Schema.fromstring(
-            self.db.iquery_readlines("show({})".format(self))[0])
+            self.db.iquery_readlines("show({})".format(self))[0][0])
 
 
 class ArrayExp(object):
@@ -699,7 +710,7 @@ class Operator(object):
             # Special case: quote string argument if not quoted
             if (pos < len(string_args) and
                     self.name in string_args[pos] and
-                    arg and
+                    len(arg) and
                     arg_fmt[0] != "'" and
                     arg_fmt[-1] != "'"):
 
@@ -809,7 +820,7 @@ class Operator(object):
                     new_schema = Schema.fromstring(
                         self.db.iquery_readlines(
                             "show('{}', 'afl')".format(
-                                str(self.args[0]).replace("'", "\\'")))[0])
+                                str(self.args[0]).replace("'", "\\'")))[0][0])
                 except requests.HTTPError as e:
                     e.args = (
                         '"temp=True" not supported for complex queries\n' +
@@ -879,7 +890,7 @@ class Operator(object):
         if self.is_lazy:
             return Schema.fromstring(
                 self.db.iquery_readlines(
-                    "show('{}', 'afl')".format(self))[0])
+                    "show('{}', 'afl')".format(self))[0][0])
 
 
 connect = DB
